@@ -37,19 +37,19 @@
 #include "castleengine.h"
 
 //-----------------------------------------------------------------------------
-typedef void (CDECL *PFNRD_CGE_Open)(unsigned uiFlags, unsigned initialWidth, unsigned initialHeight, const char *applicationConfigDirectory);
+typedef void (CDECL *PFNRD_CGE_Open)(unsigned uiFlags, unsigned initialWidth, unsigned initialHeight, unsigned uiDpi, const char *applicationConfigDirectory);
 typedef void (CDECL *PFNRD_CGE_Close)();
 typedef void (CDECL *PFNRD_CGE_GetOpenGLInformation)(char *szBuffer, int nBufSize);
 
 typedef void (CDECL *PFNRD_CGE_Resize)(unsigned uiViewWidth, unsigned uiViewHeight);
 typedef void (CDECL *PFNRD_CGE_Render)();
 typedef void (CDECL *PFNRD_CGE_SaveScreenshotToFile)(const char *szFile);
-typedef void (CDECL *PFNRD_CGE_SetLibraryCallbackProc)(TCgeLibraryCallbackProc pProc);
+typedef void (CDECL *PFNRD_CGE_SetLibraryCallbackProc)(TCgeLibraryCallback pProc);
 typedef void (CDECL *PFNRD_CGE_Update)();
 
 typedef void (CDECL *PFNRD_CGE_MouseDown)(int x, int y, bool bLeftBtn, int nFingerIdx);
 typedef void (CDECL *PFNRD_CGE_Motion)(int x, int y, int nFingerIdx);
-typedef void (CDECL *PFNRD_CGE_MouseUp)(int x, int y, bool bLeftBtn, int nFingerIdx);
+typedef void (CDECL *PFNRD_CGE_MouseUp)(int x, int y, bool bLeftBtn, int nFingerIdx, bool trackReleased);
 typedef void (CDECL *PFNRD_CGE_MouseWheel)(float zDelta, bool bVertical);
 
 typedef void (CDECL *PFNRD_CGE_KeyDown)(int eKey);
@@ -70,7 +70,7 @@ typedef void (CDECL *PFNRD_CGE_MoveViewToCoords)(float fPosX, float fPosY, float
 typedef int (CDECL *PFNRD_CGE_GetNavigationType)();
 typedef void (CDECL *PFNRD_CGE_SetNavigationType)(int eNewType);
 typedef void (CDECL *PFNRD_CGE_SetTouchInterface)(int eMode);
-typedef void (CDECL *PFNRD_CGE_SetUserInterface)(bool bAutomaticTouchInterface, int nDpi);
+typedef void (CDECL *PFNRD_CGE_SetUserInterface)(bool bAutomaticTouchInterface);
 
 typedef void (CDECL *PFNRD_CGE_SetVariableInt)(int eVar, int nValue);
 typedef int (CDECL *PFNRD_CGE_GetVariableInt)(int eVar);
@@ -171,10 +171,10 @@ void CGE_LoadLibrary()
 }
 
 //-----------------------------------------------------------------------------
-void CGE_Open(unsigned uiFlags, unsigned initialWidth, unsigned initialHeight, const char *applicationConfigDirectory)
+void CGE_Open(unsigned uiFlags, unsigned initialWidth, unsigned initialHeight, unsigned uiDpi, const char *applicationConfigDirectory)
 {
 	if (pfrd_CGE_Open!=NULL)
-		(*pfrd_CGE_Open)(uiFlags, initialWidth, initialHeight, applicationConfigDirectory);
+		(*pfrd_CGE_Open)(uiFlags, initialWidth, initialHeight, uiDpi, applicationConfigDirectory);
 }
 
 //-----------------------------------------------------------------------------
@@ -213,7 +213,7 @@ void CGE_SaveScreenshotToFile(const char *szFile)
 }
 
 //-----------------------------------------------------------------------------
-void CGE_SetLibraryCallbackProc(TCgeLibraryCallbackProc pProc)
+void CGE_SetLibraryCallbackProc(TCgeLibraryCallback pProc)
 {
 	if (pfrd_CGE_SetLibraryCallbackProc!=NULL)
 		(*pfrd_CGE_SetLibraryCallbackProc)(pProc);
@@ -241,10 +241,10 @@ void CGE_Motion(int x, int y, int nFingerIdx)
 }
 
 //-----------------------------------------------------------------------------
-void CGE_MouseUp(int x, int y, bool bLeftBtn, int nFingerIdx)
+void CGE_MouseUp(int x, int y, bool bLeftBtn, int nFingerIdx, bool trackReleased)
 {
 	if (pfrd_CGE_MouseUp!=NULL)
-		(*pfrd_CGE_MouseUp)(x, y, bLeftBtn, nFingerIdx);
+		(*pfrd_CGE_MouseUp)(x, y, bLeftBtn, nFingerIdx, trackReleased);
 }
 
 //-----------------------------------------------------------------------------
@@ -352,10 +352,10 @@ void CGE_SetTouchInterface(int /*ECgeTouchCtlInterface*/ eMode)
 }
 
 //-----------------------------------------------------------------------------
-void CGE_SetUserInterface(bool bAutomaticTouchInterface, int nDpi)
+void CGE_SetUserInterface(bool bAutomaticTouchInterface)
 {
     if (pfrd_CGE_SetUserInterface!=NULL)
-        (*pfrd_CGE_SetUserInterface)(bAutomaticTouchInterface, nDpi);
+        (*pfrd_CGE_SetUserInterface)(bAutomaticTouchInterface);
 }
 
 //-----------------------------------------------------------------------------
